@@ -9,7 +9,7 @@ setClass( "sts", representation(epoch = "numeric",  ##this slot used to be named
                                 upperbound  = "matrix",
                                 neighbourhood= "matrix",
                                 populationFrac= "matrix",
-                                map = "SpatialPolygonsDataFrame",
+                                map = "SpatialPolygons",
                                 control = "list",
 #New slots added to handle proportion time series 
                                 epochAsDate="logical",
@@ -21,6 +21,12 @@ setClass( "sts", representation(epoch = "numeric",  ##this slot used to be named
 # Definition of the stsBP class for backprojections.
 ######################################################################
 
-setClass( "stsBP", representation(ci = "array",
-                                  lambda = "array"),
-                    contains = "sts")
+setClass( "stsBP", slots = list(ci = "array", lambda = "array"),
+         contains = "sts")
+
+######################################################################
+# Definition of the stsNC class for nowcasts.
+######################################################################
+
+setClass( "stsNC", slots=list(reportingTriangle = "matrix", predPMF= "list", pi = "array", truth= "sts", delayCDF = "list", SR = "array"),
+         contains = "sts")
